/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef HDDVD_AACS_H
#define HDDVD_AACS_H

#define SHA1_LENGTH	20
#define MAC_LENGTH	16
#define VK_LENGTH	16
#define TK_LENGTH	16

//#include "aacs_crypto.hpp"
//#include "mediakeyblock.hpp"
#include "logger.hpp"
#include <string>
#include <stdint.h>

class MediaKeyBlock;
class AACSCrypto;

/**
 * AACS functionality
 */
class AACS
{
public:
	AACS(Logger* 			pLog,
		 const bool 		useDriveAuthentication,
		 const std::string& device,
		 const std::string& mountPoint);
	~AACS();

	void 			DetectMedia			();
	void 			LoadTitleKeyFile	();
	void 			ExtractTitleKeys	(uint8_t*			TKFileBuffer,
										 size_t 			TKFileSize
										);
	void			DecryptTitleKeys	(const std::string& strVolumeKey);
	int				GetKeyFromEVOB		(const std::string 	EVOBFilename);
	void			PrintTitleKeys		();
	bool			HasKey				(const int		 	keyNumber);

	void			TitleKey			(const uint8_t 		keyNumber,
										 const uint8_t* 	TitleKey);
	const uint8_t*	TitleKeyTable		() const;
	std::string		MountPoint			() const;

private:
	Logger*				m_pLog;
	MediaKeyBlock*		m_pMKB;
	AACSCrypto*			m_pCrypto;
	bool				m_useDriveAuthentication;
	std::string			m_device;
	std::string			m_mountPoint;
	bool 				m_isBRDisk;
	std::string			m_TitleKeyFilename;
	uint8_t				m_TitleKeyTable[64][16];
};

#endif
