/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef HD_DVD_ACA_FILE_H
#define HD_DVD_ACA_FILE_H

#include "arfile.hpp"

#include "logger.hpp"
#include <string>
#include <fstream>
#include <vector>

class ARFile;
struct ResourceFile;

/**
 * Handles encrypted ACA files
 */
class ACAFile
{
public:
	ACAFile(Logger* pLog);
	~ACAFile();

	unsigned long	CalcDecryptedFilesize	(std::istream& 			ACAStream);
	void 			Decrypt				(const std::string&		sourceFilename,
										 const std::string& 	destinationDir,
										 const uint8_t* const	pTitleKeyTable,
										 bool					writeToDisk = false);
	void			Open				(const std::string& 	sourceFilename,
										 const uint8_t* const 	pTitleKeyTable,
										 std::ifstream&			ACAStream,
										 bool					writeToDisk = false);
	unsigned long	ReadChunk			(unsigned long 			offset,
										 unsigned long 			chunkSize,
										 uint8_t* const 		currentChunk);
	void 			DumpFileList		();

	// accessors
	unsigned long	DecryptedFilesize	();

private:
	void 			ExtractList			(std::istream& 		ACAStream);
	void			LoadHeader			(std::istream& 		ACAStream);
	void 			ExtractARF			(ARFile&			Resource,
										 std::ifstream& 	ACAStream,
										 int				fileNo);
	void 			UpdateHeader		(int				fileNo,
										 const ARFile&		Resource);
	ARFile::MIMEType GetMIME			(const std::string&	filename);
	unsigned long	SwitchEndianLong	(unsigned long*		value);
	unsigned short 	SwitchEndianShort	(unsigned short*	value);

	Logger*						m_pLog;
	std::vector<ResourceFile*> 	m_fileList;				/**<  */
	unsigned long 				m_filesize;				/**<  */
	int 						m_noOfFiles;			/**<  */
	uint8_t*					m_pTitleKeyTable;		/**<  */
	unsigned long				m_decryptedACASize;		/**< Size of decrypted ACA file */
	uint8_t* 					m_pDecryptedACA;		/**<  */
	unsigned long				m_fp;					/**< position in final ACA */
	std::string 				m_outputDir;			/**<  */
};

#endif
