/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef HD_DVD_ADVANCED_RESOURCE_FILE_H
#define HD_DVD_ADVANCED_RESOURCE_FILE_H

#include "logger.hpp"

/**
 * Advanced Resourse File handling
 */
class ARFile
{
public:
	ARFile(Logger* pLog, uint8_t* pTitleKeyTable);
	ARFile(Logger* pLog, const std::string& filename);
	~ARFile();

	unsigned long 	ReadHeader		(std::ifstream& 		inputStream,
									 unsigned long 			offset = 0);
	unsigned long 	ReadData		(std::ifstream& 		inputStream,
									 unsigned long 			offset);
	void			ExtractResource	();
	unsigned long	ReadChunk		(unsigned long 			offset,
									 unsigned long 			chunkSize,
									 uint8_t* const 		buffer);
	void			Decrypt			(const uint8_t* const 	TitleKey);

	bool			IsEncrypted() const		{ return m_isEncrypted;		}
	bool			HasMAC() const			{ return m_hasMAC;			}
	bool			HasHash() const			{ return m_hasHash;			}
	int				TitleKeyPointer() const	{ return m_TITLE_KEY_PTR;	}
	unsigned long	RawDataSize() const		{ return m_rawDataSize;		}
	unsigned long	Filesize() const		{ return m_filesize;		}
	std::string		Filename() const		{ return m_filename;		}
	uint8_t*		ResourceFile() const	{ return m_pResourceFile;	}

	// what about MMF, LPCM?
	enum MIMEType {
		MIME_XPL 		= 0x01,	// Playlist (.xpl)
		MIME_XMF 		= 0x02,	// Manifest file (.xmf)
		MIME_XMU 		= 0x03,	// Markup file (.xmu)
		MIME_XTS 		= 0x04,	// Timing sheet (.xts)
		MIME_XAS 		= 0x05,	// Advanced subtitle (.xas)
		MIME_XSS 		= 0x06,	// Style sheet (.xss)
		MIME_JS  		= 0x07,	// ECMAScript file (.js)
		MIME_EVO 		= 0x08,	// .evo
		MIME_MAP 		= 0x09,	// Time map (.map)
		MIME_JPG 		= 0x0A,	// .jpg
		MIME_PNG 		= 0x0B,	// Portable network graphic (.png)
		MIME_MNG 		= 0x0C,	// Multiple-network-graphic (.mng)
		MIME_CVI 		= 0x0D,	// Capture image format (.cvi)
		MIME_CDW 		= 0x0E,	// Capture image format (.cdw)
		MIME_WAV 		= 0x0F,	// Windows audio format (.wav)
		MIME_XML 		= 0xFF,	// XML (.xml)
		MIME_XUL 		= 0xFF,	// XUL (.xul)
		MIME_UNKNOWN 	= 0xFF,	// unknown files (encrypted and .exe :))
		MIME_TTF 		= 0x10,	// TrueType Font (.ttf) (also reported to be 0xFF!)
		MIME_OTF 		= 0x10,	// OpenType Font (.otf)
		MIME_TTC 		= 0x10	// TrueType collection (.ttc)
	};

	enum ProtectionType
	{
		ENCRYPTED		= 0x01,
		MAC				= 0x02,
		ENCRYPTED_HASH	= 0x11,
		HASH			= 0x12,
		NON_PROTECTED	= 0x21
	};

private:
	Logger*			m_pLog;
	uint8_t*		m_pTitleKeyTable;
	uint8_t*		m_pRawData;			/**< Encrypted resouce + RFN */
	unsigned long	m_rawDataSize;
	bool			m_isEncrypted;
	bool			m_hasHash;
	bool			m_hasMAC;
	unsigned long	m_CRC32;
	ProtectionType	m_protectionType;
	int				m_TITLE_KEY_PTR;
	unsigned long	m_filesize;			// size of actual resource (no RFN)
	std::string		m_filename;			// decrypted Resource File Name (UTF-16 big endian!!!)
	uint8_t*		m_pResourceFile;	// actual ARF content
};

#endif
