/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef HDDECRYPTER_LOGGING_H
#define HDDECRYPTER_LOGGING_H

#include <string>
#include <fstream>

#define LogError(msg)	LogMessage(Logger::ERROR_MESSAGE, msg);
#define LogInfo(msg)	LogMessage(Logger::INFO_MESSAGE, msg);
#define LogNotice(msg)	LogMessage(Logger::NOTICE_MESSAGE, msg);
#define LogDebug(msg)	LogMessage(Logger::DEBUG_MESSAGE, msg);

/**
 * Takes care of logging messages
 */
class Logger
{
public:
	Logger(const std::string& logFilename);
	Logger(const std::istream& logStream);
	~Logger();

	enum logLevel
	{
		ERROR	= 1,		// log only errors
		NOTICE	= 2,		// not real error, but stuff to keep an eye on
		INFO	= 3,		// log informative messages aswell
		DEBUG	= 4			// log everything
	};

	enum messageType
	{
		ERROR_MESSAGE,
		NOTICE_MESSAGE,
		INFO_MESSAGE,
		DEBUG_MESSAGE
	};

	void LogMessage(messageType, const std::string& message);
	void SetLocation(const std::string& location);
	void SetLevel(logLevel level);

private:
	Logger(const Logger&);
	std::string	Timestamp() const;
	std::string TimeAsString() const;

	std::ofstream 	m_logfile;
	logLevel		m_level;
	std::string		m_location;
};

#endif
