/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 * ACA decrypter by Zotty
 * A program for decrypting AACS protected resource files (.aca files)
 */

#include "acadecrypter.hpp"
#include "aacs.hpp"
#include "logger.hpp"
#include <iostream>
#include <sstream>
#include <stdexcept>

void Usage(const std::string& version);

const std::string version = "v0.4";

/**
 *
 */
int main (int argc, const char* argv[])
{
	std::stringstream appString;
	appString << version << " (" << __DATE__ << " " << __TIME__ << ")";

	if (argc < 4)
	{
		Usage(appString.str());
		return 0;
	}

	// process args
	std::string keyDBFilename	= argv[1];		// keydb.xml
	std::string mountPoint		= argv[2];		// directory containing HDDVD files
	std::string filename		= argv[3];		// file to decrypt, incl full path
	std::string destinationDir	= argv[4];		// write to file

	// start logger
	//std::auto_ptr<Logger> pLog(new Logger("decryptaca.log"));
	Logger oLog("decryptaca.log");
	oLog.SetLevel(Logger::DEBUG);
	oLog.LogInfo("Decrypt ACA " + appString.str());
	oLog.LogInfo("KeyDB:\t"		+ keyDBFilename);
	oLog.LogInfo("Filename:\t"	+ filename);

	try
	{
		ACADecrypter Decrypter(oLog);
		Decrypter.SetKeyDB(keyDBFilename);
		Decrypter.SetMountPoint(mountPoint);
		Decrypter.SetWriteOutput(true);
		Decrypter.DecryptFile(filename, destinationDir);
		oLog.LogInfo("Finished decrypting");
	}
	catch (const std::exception &e)
	{
		std::stringstream errorMsg;
		errorMsg << "\n### Exception: " << e.what() << " ###";
		oLog.LogError(errorMsg.str());
	}
	catch (...)
	{
		oLog.LogError("\n### Unspecified exception occured ###");
	}

	return 0;
}

/**
 * Show usage message
 */
void Usage(const std::string& version)
{
	std::cout	<< "ACA decrypter - " << version << std::endl
				<< "by Zotty" << std::endl
				<< std::endl
				<< "Usage: \t\tdecryptaca <keydb> <mountpoint> <source> <destination>" << std::endl
				<< "Example: \t./decryptaca keydb.xml disc/ MENUS.ACA decrypted/" << std::endl
				<< std::endl;
}
