/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "logger.hpp"
#include <iostream>
#include <sstream>
#include <iomanip>
#include <ctime>

/**
 * Start logfile
 */
Logger::Logger(const std::string& logFilename)
:	m_level(INFO)
{
	m_logfile.open(logFilename.c_str());

	if (!m_logfile.is_open()) {
		std::cerr << "Error starting log " << logFilename << std::endl;
		return;
	}

	LogInfo("*** Starting log - " + TimeAsString() + " ***");
}

Logger::~Logger()
{
	if (m_logfile) {
		m_logfile.close();
	}
}

/**
 * Write message to log
 */
void Logger::LogMessage(messageType type, const std::string& message)
{
	if (!m_logfile.is_open()) {
		std::cerr << "Error writing to logfile!" << std::endl;
		return;
	}

	std::stringstream logLine;
	logLevel level = DEBUG;

	switch (type)
	{
		case ERROR_MESSAGE:
			level = ERROR;
			logLine << "E " << Timestamp() << " - " << message << std::endl;
			break;

		case NOTICE_MESSAGE:
			level = NOTICE;
			logLine << "N " << Timestamp() << " - " << message << std::endl;
			break;

		case INFO_MESSAGE:
			level = INFO;
			logLine << "I " << Timestamp() << " - " << message << std::endl;
			break;

		default:
		case DEBUG_MESSAGE:
			level = DEBUG;
			logLine << "D " << Timestamp() << " - " << message << std::endl;
			break;
	}

	if (level <= m_level)
	{
		m_logfile.write(logLine.str().c_str(), logLine.str().length());
		m_logfile.flush();
	}
}

/**
 * Create timestamp string
 */
std::string Logger::Timestamp() const
{
	time_t 		rawtime;
	struct tm*	timeinfo;

	time(&rawtime);
	timeinfo = localtime(&rawtime);
	std::stringstream timestamp;
	timestamp	<< std::setw(2) << std::setfill('0') << timeinfo->tm_hour	<< ":"
				<< std::setw(2) << std::setfill('0') << timeinfo->tm_min	<< ":"
				<< std::setw(2) << std::setfill('0') << timeinfo->tm_sec;

	return timestamp.str();
}

/**
 * Get current date and time as string
 *
 * Format: Www Mmm dd hh:mm:ss yyyy
 */
std::string Logger::TimeAsString() const
{
	time_t 		rawtime;
	struct tm*	timeinfo;

	time(&rawtime);
	timeinfo = localtime(&rawtime);
	std::string timeString = asctime(timeinfo);

	return timeString.substr(0, timeString.length() - 1);
}

//---------------

void Logger::SetLocation(const std::string& location)
{
	m_location = location;
}
void Logger::SetLevel(logLevel level)
{
	std::stringstream message;
	message << "Loglevel is now set to " << level;
	LogNotice(message.str());
	m_level = level;
}
