/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "util.hpp"
#include <iostream>
#include <string>
#include <sstream>
#include <iomanip>
#include <exception>
#include <algorithm>	// tokenizer

/**
 * Convert array to string of hex values
 */
void Utils::ToHexString(unsigned char const* ucArray, std::string& strHex, const unsigned short usLength)
{
	std::stringstream ssKey;

	for (int i = 0; i < usLength; i++)
	{
		ssKey << std::hex << std::uppercase << std::setw(2) << std::setfill('0') << (unsigned short)ucArray[i];
	}
	
	strHex = ssKey.str();
}

/**
 * Convert array to string and return it
 */
std::string Utils::ToHexString(uint8_t const* ucArray, const unsigned short usLength)
{
	std::stringstream ssKey;

	for (int i = 0; i < usLength; i++)
	{
		ssKey << std::hex << std::uppercase << std::setw(2) << std::setfill('0') << (unsigned short)ucArray[i];
	}
	
	return ssKey.str();
}

/**
 * Convert string to array
 */
void Utils::HexStringToByteArray(const std::string& strHex, unsigned char *ucArray)
{
	for (unsigned short i = 0; i < strHex.length() / 2; i++)
	{
		ucArray[i] = 0;
		int shift  = 0;
		
		// 1 byte in array is 2 chars from string
		for (unsigned short j = 0; j < 2; j++)
		{
			char cCurrent = strHex[i * 2 + j];
			
			j == 0 ? shift = 4 : shift = 0;
			
			if (cCurrent >= '0' && cCurrent <= '9')
			{
				ucArray[i] += (cCurrent - '0') << shift;
			}
			else if (cCurrent >= 'A' && cCurrent <= 'F')
			{
				ucArray[i] += (cCurrent - 'A' + 10) << shift;
			}
			else
			{
				std::cerr << "Invalid character" << std::endl;
			}
		}
	}
}

/**
 * XOR
 */
void Utils::Xor(unsigned char (&buff)[16], unsigned char const* chain)
{
	for (int i = 0; i < sizeof(buff); i++) {
		buff[i] ^= *(chain++);
	}
}

/**
 * Strip leading/trailing spaces
 */
void Utils::Trim(std::string& str)
{
	std::string::size_type pos = str.find_last_not_of(' ');
	pos = str.find_last_not_of('\t', pos);	// filter tabs (keydb.cfg)
	
	if( pos != std::string::npos )
	{
		str.erase(pos + 1);
		pos = str.find_first_not_of(' ');
		if(pos != std::string::npos) str.erase(0, pos);
	}
	else
		str.erase(str.begin(), str.end());
}

/**
 * Tokenize string
 * leading/trailing spaces are trimmed
 */
void Utils::Tokenize(const std::string& str,
                      std::vector<std::string>& tokens,
                      const std::string& delimiters = " ")
{
    // Skip delimiters at beginning.
    std::string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    std::string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (std::string::npos != pos || std::string::npos != lastPos)
    {
    	// Found a token, trim and add it to the vector.
    	std::string tmp = str.substr(lastPos, pos - lastPos);
    	Trim(tmp);
        tokens.push_back(tmp);
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}

/**
 * Add slash after string
 */
void Utils::AddSlash(std::string& path)
{
	if (path.empty())
		return;
	
	// add trailing slash
	if (path.at(path.length()-1) != '/')
		path += '/';
}
