/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 * ECDSA auto cleanup stuff (aka simple smart pointers)
 */

#include <openssl/ecdsa.h>

static EC_GROUP* EC_GROUP_NULL = 0;

struct EC_GROUP_auto_ptr
{
	EC_GROUP_auto_ptr(EC_GROUP* pGroup) : m_pGroup(pGroup) {}
	~EC_GROUP_auto_ptr() { EC_GROUP_free(m_pGroup); }

	EC_GROUP* operator->() const { return m_pGroup; }
    EC_GROUP& operator*() const { return *m_pGroup; }
	
	EC_GROUP* m_pGroup;
	
	// give up ownership
	EC_GROUP* release()
	{
		EC_GROUP* pTmp(m_pGroup);
		EC_GROUP_free(m_pGroup);
		return pTmp;
	}
};

struct EC_POINT_auto_ptr
{
	EC_POINT_auto_ptr(EC_POINT* pPoint) : m_pPoint(pPoint) {}
	~EC_POINT_auto_ptr() { EC_POINT_free(m_pPoint); }
	
	EC_POINT* m_pPoint;
};

struct BN_auto_ptr
{
	BN_auto_ptr(BIGNUM* pBN) : m_pBN(pBN) {}
	//~BN_auto_ptr() { BN_free(m_pBN); }
	~BN_auto_ptr() { BN_clear_free(m_pBN); }
	
	BIGNUM* m_pBN;
/*	
	BIGNUM* release()
	{
		BIGNUM* pTmp(m_pBN);
		BN_free(m_pBN);
		return pTmp;
	}
*/
};

struct BN_CTX_auto_ptr
{
	BN_CTX_auto_ptr(BN_CTX* pCTX) : m_pCTX(pCTX) {}
	~BN_CTX_auto_ptr() { BN_CTX_free(m_pCTX); }
	
	BN_CTX* m_pCTX;
};

struct EC_KEY_auto_ptr
{
	EC_KEY_auto_ptr(EC_KEY* pKey) : m_pKey(pKey) {}
	~EC_KEY_auto_ptr() { EC_KEY_free(m_pKey); }
	
	EC_KEY* m_pKey;
};

struct ECDSA_SIG_auto_ptr
{
	ECDSA_SIG_auto_ptr(ECDSA_SIG* pSig) : m_pSig(pSig) {}
	~ECDSA_SIG_auto_ptr() { ECDSA_SIG_free(m_pSig); }
	
	ECDSA_SIG* m_pSig;
};
