/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "file.hpp"
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdexcept>

/**
 *
 */
File::File(std::string strFileName)
:	m_strFileName(strFileName),
	m_ui64FileSize(0),
	m_bIsDirectory(false)
{
	if (strFileName.empty()) {
		/** @todo Throw proper exception */
		throw;
	}
}

File::File(std::string strPath, std::string strFileName, uint64_t ui64FileSize, bool bIsDirectory)
:	m_strPath(strPath),
	m_strFileName(strFileName),
	m_ui64FileSize(ui64FileSize),
	m_bIsDirectory(bIsDirectory)
{
}

/**
 * Load entire file into buffer
 */
unsigned char* File::LoadFileIntoMemory()
{
	if (0LL == m_ui64FileSize) CalcFileSize();

	unsigned char *pBuffer;
	
	try
	{
		pBuffer = new unsigned char[m_ui64FileSize];
	}
	catch (std::bad_alloc &e)
	{
		std::stringstream ssErr;
		ssErr << "LoadFileIntoMemory: " << e.what();
		throw std::runtime_error(ssErr.str());
	}
		
	std::ifstream fis(m_strFileName.c_str(), std::ios_base::binary | std::ios_base::in);
	
	// save current state and set
	std::ios::iostate oldExceptions = fis.exceptions();
	fis.exceptions(std::ios::failbit | std::ios::badbit | std::ios::eofbit);
	
	try 
	{
		fis.read((char *)pBuffer, m_ui64FileSize);
	}
	catch (...)
	{
		fis.exceptions(oldExceptions);		// restore
		throw;
	}
	
	fis.close();
	fis.exceptions(oldExceptions);		// restore
	
	return pBuffer;
}

/**
 * Get filesize, OS independant
 */
void File::CalcFileSize()
{
	//std::cout << "Calculating filesize" << std::endl;
	
	std::ifstream fis(m_strFileName.c_str(), std::ios_base::binary | std::ios_base::in);
	//if (!fis.good() || fis.eof() || !fis.is_open()) {
	if (!fis) {
		std::cerr << "Error opening file to calc filesize" << std::endl;
		return; 
	}
	
	fis.seekg(0, std::ios_base::beg);
	std::ifstream::pos_type begin_pos = fis.tellg();
	fis.seekg(0, std::ios_base::end);
	m_ui64FileSize = static_cast<uint64_t>(fis.tellg() - begin_pos);
}
