/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef AACS_HDDRIVE_H
#define AACS_HDDRIVE_H

#include "ioctl.h"
#include "aacs_crypto.hpp"
#include "logger.hpp"
#include <string>
#include <inttypes.h>

/**
 * Handles HD DVD drive stuff
 */
class HDDrive
{
public:
	HDDrive(Logger* pLog, const std::string& device, AACSCrypto* pCrypto);
	~HDDrive();

	void 	Authenticate			();

	//
	bool		IsAuthenticated		();
	uint8_t*	VolumeID			();

private:
	void	ReadHostData			();
	int 	ReadHostDataFromDisk	();
	bool 	AACSAuthentication		(uint8_t 		AGID);
	void 	VerifyVolumeID			();
	bool 	IsVolumeIDValid			();
	bool 	InvalidateAGID			(uint8_t 		AGID);
	int8_t	ReportAGID				();
	bool 	ReportDriveCertificate	(uint8_t 		AGID,
									 uint8_t* 		Nonce,
									 uint8_t*		Certificate
									);
	bool 	ReportDriveKey			(uint8_t 		AGID,
									 uint8_t* 		Point,
									 uint8_t* 		Signature
									);
	bool 	SendHostCertificate		(uint8_t 		AGID,
									 uint8_t* 		Nonce,
									 uint8_t* 		Certificate
									);
	bool 	SendHostKey				(uint8_t 		AGID,
									 uint8_t* 		Point,
									 uint8_t* 		Signature
									);
	bool	ReadVolumeID			(uint8_t 		AGID,
									 uint8_t* 		VolumeID,
									 uint8_t* 		MAC
									);
	bool 	ReadVolumeID_hack		(uint8_t 		AGID,
									 uint8_t* 		VolumeID,
									 uint8_t* 		MAC
									);
	//bool 	GetFirmware				(uint8_t* 		firmware);
	bool 	GetDriveInfo			();

	bool CommandDfMemset	(uint32_t 		offset,
							 size_t 		size,
							 uint16_t 		value);
	bool EnableCommandDF	();
	bool DisableCommandDF	();
	bool ReportKey			(uint8_t* 		buffer,
							 uint8_t 		AGID,
							 uint8_t 		KeyFormat,
							 short 			length);
	bool SendKey			(uint8_t* 		buffer,
							 uint8_t 		AGID,
							 uint8_t 		KeyFormat,
							 short 			length);
	bool ReadDiskStructure	(uint8_t* 		buffer,
							 uint8_t 		AGID,
							 short 			length);
	bool GetConfiguration	(uint8_t* 		buffer,
							 uint16_t 		feature,
							 short 			length);
	bool Inquiry			(uint8_t* 		buffer,
							 short 			length);
	bool SendDiagnostic		(uint8_t* 		buffer,
							 short 			length);

	Logger*			m_pLog;
	AACSCrypto*		m_pCrypto;
	drive_handle 	m_hHandle;
	uint8_t			m_cmd[CDROM_PACKET_SIZE];
	bool			m_isAuthenticated;
	uint8_t 		m_HostNonce[20];			// hex
	uint8_t 		m_HostCertificate[92];		// hex
	uint8_t 		m_HostKeyPoint[40];			// hex
	uint8_t 		m_HostKey[20];				// hex
	std::string 	m_HostPrivateKey;			// ascii
	uint8_t 		m_DriveNonce[20];
	uint8_t 		m_DriveCertificate[92];
	uint8_t 		m_DriveKeyPoint[40];
	uint8_t 		m_DriveKeySignature[40];
	uint8_t 		m_HostKeySignature[40];
	uint8_t 		m_BusKey[16];
	uint8_t 		m_VolumeID[16];
	uint8_t 		m_VolumeIDMAC[16];
	std::string		m_vendorID;
	std::string		m_productID;
	std::string		m_revision;
	std::string		m_date;
};

#endif
