/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ioctl.h"
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#include <errno.h>

/**
 * Open drive handle
 */
drive_handle open_drive(char * device_file) 
{
	return open(device_file, O_RDONLY | O_NONBLOCK);
}

/**
 * Close drive handle
 */
int close_drive(drive_handle hHandle) 
{
	return close(hHandle);
}

/**
 * Send command to drive
 */
int send_cmd(drive_handle hHandle, unsigned char *cmd, unsigned char *buf, size_t send, size_t recv) 
{
	struct cdrom_generic_command cgc;
	struct request_sense sense;

	if (send > 0 && recv > 0)
	{
		return -1;
	}

	// copy into command struct
	memset(&cgc, 0, sizeof(cgc));
	memcpy(cgc.cmd, cmd, CDROM_PACKET_SIZE);
	cgc.sense = &sense;
	cgc.timeout = 1000;

	cgc.buffer = (unsigned char *)buf;
	
	// write to drive
	if(send > 0) 
	{
		cgc.buflen = send;
		cgc.data_direction = CGC_DATA_WRITE;
	} 
	
	// read from drive
	else if(recv > 0) 
	{
		cgc.buflen = recv;
		cgc.data_direction = CGC_DATA_READ;
	} 
	else 
	{
		cgc.buflen = 0;
		cgc.data_direction = CGC_DATA_NONE;
	}

	int ret = ioctl(hHandle, CDROM_SEND_PACKET, &cgc);
	if (ret < 0) 
	{
		printf("ioctl failed: %d errno %d stat %d\n", ret, errno, cgc.stat);
		return ret;
	}

	return ret;
}
