/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef HDDECRYPTER_MEDIAKEYBLOCK_H
#define HDDECRYPTER_MEDIAKEYBLOCK_H

#include "logger.hpp"
#include <string>
#include <stdint.h>
#include <map>
#include <vector>

/**
 * MKB stuff
 */
class MediaKeyBlock
{
public:
	MediaKeyBlock(Logger* pLog);
	~MediaKeyBlock();

	void Load(const std::string strMountPoint);

	bool		IsProcessed		();
	uint32_t	Version			();
	uint32_t	Type			();
	uint8_t*	VerificationData();
	uint8_t*	MediaKeyData	(const int iRecordNo);

	enum eRecordType
	{
		RECORD_UNKNOWN		= 0xFF,
		RECORD_VERSION		= 0x10,		// Type and version record (only 1)
		RECORD_HOST_RL		= 0x21,		// Host Revocation List record (only 1)
		RECORD_DRIVE_RL		= 0x20,		// Drive Revocation List record (only 1)
		RECORD_VERIFY_MK	= 0x81,		// Verify Media Key record (only 1)
		RECORD_EXPL_SUBSET	= 0x04, 	// Explicit Subset-Difference record
		RECORD_SUBSET_INDEX	= 0x07,		// Subset-Difference Index record
		RECORD_MK_DATA		= 0x05,		// Media Key Data record
		RECORD_MKB_END		= 0x02		// End of Media Key Block record (only 1)
	};

private:
	void 	ExtractVersion					(uint8_t* buffer);
	void 	ExtractVerificationData			(uint8_t* buffer);
	void 	ExtractMediaKeyData				(uint8_t* buffer, const uint32_t uiLength);
	void 	ExtractExplicitSubsetDifferences(uint8_t* buffer);

	Logger*		m_pLog;
	bool		m_bIsProcessed;
	uint32_t	m_uiRecordLength;
	uint8_t*	m_pucMKBFile;
	uint32_t	m_uiMKBSize;
	uint32_t	m_u32Type;
	uint32_t	m_u32Version;
	uint8_t		m_VerificationData[16];
	uint8_t*	m_MediaKeyDataTable;
	//uint8_t*	m_UmaskTable;
	//uint32_t*	m_UVnumberTable;

	std::map<int, uint8_t>	m_mUmasks;
	std::map<int, uint32_t>	m_mUVnumbers;
};

#endif
